"""
A HTTP service that relays requests to a vSphere API host. Based on the Flask_
framework.

.. _Flask: http://flask.pocoo.org/

Service API
===========

This section describes the API used to communicate with the HTTP service.

Status Codes
++++++++++++

All API endpoints can return one of the following HTTP status codes.

- **200** if the request is successful.
- **400** if the request could not be processed.
- **403** if the request could not be authorized.
- **404** if a requested entity could not be found.
- **500** if an internal server error happened.

Errors
++++++

Errors are communicated with the appropriate HTTP status code and a response
JSON object on the following form:

::

    {
        "errors": [
            {
                "detail": "<error message>",
                "status": "<http status message>"
            },
            ...
        ]
    }

GET /v1/query
+++++++++++++

Retrieves a set of performance data.

URL parameters
--------------

- **host**
    Hostname or IP address of the vSphere API.

- **port**
    Port number of the vSphere API.

- **ignore_ssl**
    Set to `true` to not use SSL.

- **username**
    Username for the vSphere API.

- **password**
    Password for the vSphere API.

- **datacenter**
    Name of the requested datacenter. Used as filter if any of the `cluster`,
    `host_system` or `vm` parameters are also given.

- **cluster**
    Name of the requested cluster. Used as filter if any of the `host_system`
    or `vm` parameters are also given.

- **host_system**
    Name of the requested ESXi host.

- **vm**
    Name of the requested virtual machine.

- **counter**
    Name of the requested performance counter.

    The key string is on the form: `<group>.<counter>.<rollup_type>`

    E.g. `cpu.usage.average`, `mem.usage.average` or `disk.write.average`.

- **instance**
    The counter instance to use. Default is the "" instance.

- **interval**
    Interval ID of the returned performance data. This is the length of
    the interval in seconds.

Response Data
-------------

With a status code of 200 the response body is a JSON object on the following
form:

::

    {
        "data": [
            {
                "counter": "<counter>",
                "entity_name": "<name>",
                "unit": "<unit>",
                "value": <value>
            },
            ...
        ]
    }

GET /v1/list-counters
+++++++++++++++++++++

Retrieves a list of performance counters available for the given entity and
interval.

URL parameters
--------------

- **host**
    Hostname or IP address of the vSphere API.

- **port**
    Port number of the vSphere API.

- **ignore_ssl**
    Set to `true` to not use SSL.

- **username**
    Username for the vSphere API.

- **password**
    Password for the vSphere API.

- **datacenter**
    Name of the requested datacenter. Used as filter if any of the `cluster`,
    `host_system` or `vm` parameters are also given.

- **cluster**
    Name of the requested cluster. Used as filter if any of the `host_system`
    or `vm` parameters are also given.

- **host_system**
    Name of the requested ESXi host.

- **vm**
    Name of the requested virtual machine.

- **interval**
    Interval ID for which to return available counters.

Response Data
-------------

With a status code of 200 the response body is a JSON object on the following
form:

::

    {
        "data": {
            "<counter-1>": {
                "name": "<name>",
                "id": "<id>",
                "unit": "<unit>",
                "summary": "<summary>",
                "instances": [
                    "<instance-1>",
                    ...
                ]
            },
            ...
        }
    }

GET /v1/list-all-counters
+++++++++++++++++++++++++

Retrieves a list of all performance counters known by the vSphere API.

URL parameters
--------------

- **host**
    Hostname or IP address of the vSphere API.

- **port**
    Port number of the vSphere API.

- **ignore_ssl**
    Set to `true` to not use SSL.

- **username**
    Username for the vSphere API.

- **password**
    Password for the vSphere API.

Response Data
-------------

With a status code of 200 the response body is a JSON object on the following
form:

::

    {
        "data": {
            "<counter-1>": {
                "name": "<name>",
                "id": "<id>",
                "unit": "<unit>",
                "summary": "<summary>"
            },
            ...
        }
    }
"""
