from os.path import abspath, dirname, join

from flask import Flask
from flask_caching import Cache

from check_vmware.service.routes import routes
from check_vmware.service.errorhandler import errorhandler


def create_app(config={}):
    """
    Create and configure a new Flask app instance.
    """

    app = Flask(
        __name__,
        root_path=abspath(dirname(__file__))
    )

    app.config.from_pyfile(join(app.root_path, 'config.py'))
    app.config.from_envvar('CHECK_VMWARE_CONFIG', silent=True)
    app.config.update(config)

    app.register_blueprint(routes)
    app.register_blueprint(errorhandler)

    app.cache = Cache(config={
        'CACHE_TYPE': (
            'flask_caching.backends.' + app.config.get('CACHE_TYPE', 'redis')
        ),
        'CACHE_KEY_PREFIX': app.config.get('CACHE_KEY_PREFIX', ''),
        'CACHE_DEFAULT_TIMEOUT': app.config.get('CACHE_TTL', 5 * 60)
    })
    app.cache.init_app(app)

    # Import VSAN extensions before anything else starts using pyVmomi
    from pyVmomi import vim  # NOQA

    if app.config.get('VSAN_SDK_ENABLED'):
        import vsan_sdk.vsanmgmtObjects  # NOQA

    return app
