"""
This file provides the base configuration for the service.

The service can be further configured by assigning a configuration file to the
`CHECK_VMWARE_CONFIG` environment variable before starting the service.

See http://flask.pocoo.org/docs/0.12/config/ for documentation on the available
Flask configuration.

See http://pythonhosted.org/Flask-Caching/#configuring-flask-caching for
documentation on the available Flask-Caching configuration.
"""

#: The type of cache used. Use 'redis' for production mode. Can be changed to
#: 'simple' during development.
CACHE_TYPE = 'redis'

#: The Redis host used.
CACHE_REDIS_HOST = '127.0.0.1'

#: The Redis port used.
CACHE_REDIS_PORT = 6379

#: The Redis password used. Set to None for no authentication.
CACHE_REDIS_PASSWORD = None

#: The Redis DB used (zero-based number index).
CACHE_REDIS_DB = 0

#: The time to live for cached items.
CACHE_TTL = 15 * 60

#: The prefix used for cache keys to ensure they are unique in Redis.
CACHE_KEY_PREFIX = 'vmware_monitor'

#: Enable/disable debug mode.
DEBUG = False

#: Explicitly disable exception propagation, instead of changing it with
#: DEBUG mode. Error handling does not support this being True.
PROPAGATE_EXCEPTIONS = False

#: Load the VSAN SDK extension that patches pyVmomi. Required for some VSAN
#: checks.
VSAN_SDK_ENABLED = False

# SERVERS can be set to a Python dict containing the authentication details
# for the vSphere servers the service should know about. Hosts that are not
# configured here will require the authentication details as part of the
# command line arguments to the check_vmware command.
# SERVERS = {
#     'host': {
#         'username': 'username',
#         'password': 'password',
#         'ignore_ssl': False
#     }
# }
