from flask import Blueprint, jsonify

from check_vmware.service.models import (
    ListAllCountersParams,
    ListCountersParams,
    QueryParams,
    validate
)
from check_vmware.vsphere import connect


routes = Blueprint("routes", __name__)


@routes.route('/v1/query')
@validate(QueryParams)
def query(params):
    """
    Endpoint handler for performance data requests.
    """
    connection = _get_connection(params)
    flags = set()
    if params.regex:
        flags.add('regex')
    if params.blacklist:
        flags.add('blacklist')
    response = connection.query(
        counter_name=params.counter,
        entity_path=params.entity_path,
        interval=params.interval,
        instance=params.instance,
        flags=flags
    )
    return jsonify(response)


@routes.route('/v1/list-counters')
@validate(ListCountersParams)
def list_counters(params):
    """
    Endpoint handler for listing all counters known by a managed entity.
    """
    connection = _get_connection(params)
    response = connection.list_counters(
        entity_path=params.entity_path,
        interval=params.interval
    )
    return jsonify(dict(data=response))


@routes.route('/v1/list-all-counters')
@validate(ListAllCountersParams)
def list_all_counters(params):
    """
    Endpoint handler for listing all counters known by the vSphere server.
    """
    connection = _get_connection(params)
    response = connection.list_all_counters()
    return jsonify(dict(data=response))


def _get_connection(params):
    return connect(
        params.host,
        params.port,
        params.username,
        params.password,
        params.ignore_ssl
    )
