from check_vmware.vsphere.helpers import (
    implemented_for,
    function_properties,
    matches
)
from check_vmware.vsphere.exceptions import VsphereNotFound


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['host'])
def status(connection, entity, flags, service_name=None):
    """
    Return a list of configured services and whether they're running or not.
    The returned state is one of: `ok` or `warning`.
    """
    services = entity.configManager.serviceSystem.serviceInfo.service
    items = []
    for service in services:
        if not matches(service.key, service_name, flags):
            continue
        state = 'ok' if service.running else 'warning'
        items.append({
            'name': service.key,
            'value': 'running' if service.running else 'not running',
            'state': state
        })
    if service_name and not items:
        raise VsphereNotFound('services', service_name)
    return items
