from check_vmware.vsphere.helpers import (
    implemented_for,
    matches,
    function_properties
)
from check_vmware.vsphere.exceptions import VsphereNotFound


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['datacenter', 'cluster', 'host'])
def usage(_conn, entity, flags, datastore_name=None):
    """
    Return the datastore used space percentage of the given entity. Specify
    <datastore_name> to include only matching adapters. You may specify a
    comma-separated list of datastores.
    """
    return _usage(entity, flags, datastore_name, unit='%', free=False)


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['datacenter', 'cluster', 'host'])
def usagegb(_conn, entity, flags, datastore_name=None):
    """
    Return the datastore used space of the given entity. Specify
    <datastore_name> to include only matching adapters. You may specify a
    comma-separated list of datastores.
    """
    return _usage(entity, flags, datastore_name, unit='GB', free=False)


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['datacenter', 'cluster', 'host'])
def free(_conn, entity, flags, datastore_name=None):
    """
    Return the datastore free space percentage of the given entity. Specify
    <datastore_name> to include only matching adapters. You may specify a
    comma-separated list of datastores.
    """
    return _usage(entity, flags, datastore_name, unit='%', free=True)


@function_properties(counter_flags=['regex', 'blacklist'])
@implemented_for(['datacenter', 'cluster', 'host'])
def freegb(_conn, entity, flags, datastore_name=None):
    """
    Return the datastore free space of the given entity. Specify
    <datastore_name> to include only matching adapters. You may specify a
    comma-separated list of datastores.
    """
    return _usage(entity, flags, datastore_name, unit='GB', free=True)


def _usage(entity, flags, datastore_name, unit='%', free=False):
    states = []
    for datastore in entity.datastore:
        if not matches(datastore.name, datastore_name, flags):
            continue
        capacity = datastore.summary.capacity
        free_space = datastore.summary.freeSpace
        byte_count = free_space if free else capacity - free_space
        if unit == '%':
            value = round(float(byte_count) * 100 / capacity, 2)
            detail = u'{}{} / {}{}'.format(
                round(float(byte_count) / 1024 ** 3, 2),
                'GB',
                round(float(capacity) / 1024 ** 3, 2),
                'GB'
            )
        elif unit == 'GB':
            value = round(float(byte_count) / 1024 ** 3, 2)
            detail = u'{}% of {}{}'.format(
                round(float(byte_count) * 100 / capacity, 2),
                round(float(capacity) / 1024 ** 3, 2),
                'GB'
            )
        states.append({
            'name': datastore.name,
            'value': value,
            'unit': unit,
            'detail': detail
        })
    if not states:
        raise VsphereNotFound('datastores', datastore_name)
    return states
